@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include "util.h"
-/*@unused@*/ RCSID("$Id: expr.c 1895 2007-07-14 05:31:08Z peter $");
+/*@unused@*/ RCSID("$Id: expr.c 2148 2008-10-08 07:42:56Z peter $");
 
 #include "libyasm-stdint.h"
 #include "coretype.h"
@@ -43,6 +43,9 @@
 #include "arch.h"
 
 
+static /*@only@*/ yasm_expr *expr_level_op
+    (/*@returned@*/ /*@only@*/ yasm_expr *e, int fold_const,
+     int simplify_ident, int simplify_reg_mul);
 static int expr_traverse_nodes_post(/*@null@*/ yasm_expr *e,
                                     /*@null@*/ void *d,
                                     int (*func) (/*@null@*/ yasm_expr *e,
@@ -116,7 +119,7 @@ yasm_expr_create(yasm_expr_op op, yasm_expr__item *left,
 
     ptr->line = line;
 
-    return ptr;
+    return expr_level_op(ptr, 1, 1, 0);
 }
 /*@=compmempass@*/
 
@@ -738,7 +741,7 @@ expr_level_op(/*@returned@*/ /*@only@*/ yasm_expr *e, int fold_const,
      * ordering as was present originally.
      * Combine integer terms as necessary.
      */
-    for (i=e->numterms-1, o=level_numterms-1; i>=0; i--) {
+    for (i=fold_numterms-1, o=level_numterms-1; i>=0; i--) {
         if (e->terms[i].type == YASM_EXPR_EXPR &&
             e->terms[i].data.expn->op == e->op) {
             /* bring up subexpression */
@@ -779,7 +782,8 @@ expr_level_op(/*@returned@*/ /*@only@*/ yasm_expr *e, int fold_const,
             if (i == first_int_term)
                 first_int_term = o;
             o--;
-        }
+        } else
+            o--;
     }
 
     /* Simplify identities, make IDENT if possible, and save to e->numterms. */
