@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: file.c 2004 2007-10-16 07:40:21Z peter $");
+/*@unused@*/ RCSID("$Id: file.c 2146 2008-10-08 07:37:52Z peter $");
 
 /* Need either unistd.h or direct.h (on Windows) to prototype getcwd() */
 #if defined(HAVE_UNISTD_H)
@@ -247,52 +247,18 @@ yasm__getcwd(void)
     return buf;
 }
 
-/* FIXME: dumb way for now */
 char *
-yasm__abspath_unix(const char *path)
+yasm__abspath(const char *path)
 {
     char *curdir, *abspath;
-    static const char pathsep[2] = "/";
 
     curdir = yasm__getcwd();
-
-    abspath = yasm_xmalloc(strlen(curdir) + strlen(path) + 2);
-    strcpy(abspath, curdir);
-    strcat(abspath, pathsep);
-    strcat(abspath, path);
-
+    abspath = yasm__combpath(curdir, path);
     yasm_xfree(curdir);
 
     return abspath;
 }
 
-/* FIXME: dumb way for now */
-char *
-yasm__abspath_win(const char *path)
-{
-    char *curdir, *abspath, *ch;
-    static const char pathsep[2] = "\\";
-
-    curdir = yasm__getcwd();
-
-    abspath = yasm_xmalloc(strlen(curdir) + strlen(path) + 2);
-    strcpy(abspath, curdir);
-    strcat(abspath, pathsep);
-    strcat(abspath, path);
-
-    yasm_xfree(curdir);
-
-    /* Replace all / with \ */
-    ch = abspath;
-    while (*ch) {
-        if (*ch == '/')
-            *ch = '\\';
-        ch++;
-    }
-
-    return abspath;
-}
-
 char *
 yasm__combpath_unix(const char *from, const char *to)
 {
