@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include "util.h"
-/*@unused@*/ RCSID("$Id: symrec.c 1893 2007-07-14 03:11:32Z peter $");
+/*@unused@*/ RCSID("$Id: symrec.c 2143 2008-10-08 07:29:43Z peter $");
 
 #include <limits.h>
 
@@ -453,6 +453,21 @@ yasm_symrec_get_name(const yasm_symrec *sym)
     return sym->name;
 }
 
+char *
+yasm_symrec_get_global_name(const yasm_symrec *sym, const yasm_object *object)
+{
+    if (sym->visibility & (YASM_SYM_GLOBAL|YASM_SYM_COMMON|YASM_SYM_EXTERN)) {
+        char *name = yasm_xmalloc(strlen(object->global_prefix) +
+                                  strlen(sym->name) +
+                                  strlen(object->global_suffix) + 1);
+        strcpy(name, object->global_prefix);
+        strcat(name, sym->name);
+        strcat(name, object->global_suffix);
+        return name;
+    }
+    return yasm__xstrdup(sym->name);
+}
+
 yasm_sym_vis
 yasm_symrec_get_visibility(const yasm_symrec *sym)
 {
