@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: x86arch.c 2051 2008-04-11 09:29:58Z peter $");
+/*@unused@*/ RCSID("$Id: x86arch.c 2144 2008-10-08 07:33:10Z peter $");
 
 #include <libyasm.h>
 
@@ -449,15 +449,15 @@ x86_reg_print(yasm_arch *arch, uintptr_t reg, FILE *f)
         "r8b", "r9b", "r10b", "r11b", "r12b", "r13b", "r14b", "r15b"
     };
     static const char *name16[] = {
-        "ax", "cx", "dx", "bx", "sp", "bp", "si", "di"
+        "ax", "cx", "dx", "bx", "sp", "bp", "si", "di",
         "r8w", "r9w", "r10w", "r11w", "r12w", "r13w", "r14w", "r15w"
     };
     static const char *name32[] = {
-        "eax", "ecx", "edx", "ebx", "esp", "ebp", "esi", "edi"
+        "eax", "ecx", "edx", "ebx", "esp", "ebp", "esi", "edi",
         "r8d", "r9d", "r10d", "r11d", "r12d", "r13d", "r14d", "r15d"
     };
     static const char *name64[] = {
-        "rax", "rcx", "rdx", "rbx", "rsp", "rbp", "rsi", "rdi"
+        "rax", "rcx", "rdx", "rbx", "rsp", "rbp", "rsi", "rdi",
         "r8", "r9", "r10", "r11", "r12", "r13", "r14", "r15"
     };
 
