@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: stabs-dbgfmt.c 1974 2007-09-21 16:37:01Z peter $");
+/*@unused@*/ RCSID("$Id: stabs-dbgfmt.c 2010 2007-11-14 08:33:32Z peter $");
 
 #include <libyasm.h>
 
@@ -331,7 +331,7 @@ stabs_dbgfmt_generate(yasm_object *object, yasm_linemap *linemap,
     info.errwarns = errwarns;
     info.lastline = 0;
     info.stabcount = 0;
-    info.stab = yasm_object_get_general(object, ".stab", 0, 4, 0, 0, &new, 0);
+    info.stab = yasm_object_get_general(object, ".stab", 4, 0, 0, &new, 0);
     if (!new) {
         yasm_bytecode *last = yasm_section_bcs_last(info.stab);
         if (last == NULL) {
@@ -347,7 +347,7 @@ stabs_dbgfmt_generate(yasm_object *object, yasm_linemap *linemap,
     }
 
     info.stabstr =
-        yasm_object_get_general(object, ".stabstr", 0, 1, 0, 0, &new, 0);
+        yasm_object_get_general(object, ".stabstr", 1, 0, 0, &new, 0);
     if (!new) {
         yasm_bytecode *last = yasm_section_bcs_last(info.stabstr);
         if (last == NULL) {
