@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: bin-objfmt.c 2035 2008-02-09 03:35:07Z peter $");
+/*@unused@*/ RCSID("$Id: bin-objfmt.c 2144 2008-10-08 07:33:10Z peter $");
 
 #include <libyasm.h>
 
@@ -663,6 +663,7 @@ map_symrec_output(yasm_symrec *sym, void *d)
     map_output_info *info = (map_output_info *)d;
     const yasm_expr *equ;
     /*@dependent@*/ yasm_bytecode *precbc;
+    /*@only@*/ char *name = yasm_symrec_get_global_name(sym, info->object);
 
     assert(info != NULL);
 
@@ -673,26 +674,27 @@ map_symrec_output(yasm_symrec *sym, void *d)
         yasm_intnum_set(info->intn, yasm_expr_get_intnum(&realequ, 0));
         yasm_expr_destroy(realequ);
         map_print_intnum(info->intn, info);
-        fprintf(info->f, "  %s\n", yasm_symrec_get_name(sym));
+        fprintf(info->f, "  %s\n", name);
     } else if (yasm_symrec_get_label(sym, &precbc) &&
                yasm_bc_get_section(precbc) == info->section) {
         bin_section_data *bsd =
             yasm_section_get_data(info->section, &bin_section_data_cb);
 
         /* Real address */
-        yasm_intnum_set_uint(info->intn, precbc->offset);
+        yasm_intnum_set_uint(info->intn, yasm_bc_next_offset(precbc));
         yasm_intnum_calc(info->intn, YASM_EXPR_ADD, bsd->istart);
         map_print_intnum(info->intn, info);
         fprintf(info->f, "  ");
 
         /* Virtual address */
-        yasm_intnum_set_uint(info->intn, precbc->offset);
+        yasm_intnum_set_uint(info->intn, yasm_bc_next_offset(precbc));
         yasm_intnum_calc(info->intn, YASM_EXPR_ADD, bsd->ivstart);
         map_print_intnum(info->intn, info);
 
         /* Name */
-        fprintf(info->f, "  %s\n", yasm_symrec_get_name(sym));
+        fprintf(info->f, "  %s\n", name);
     }
+    yasm_xfree(name);
     return 0;
 }
 
