@@ -26,7 +26,7 @@
  */
 
 #include <util.h>
-/*@unused@*/ RCSID("$Id: elf-x86-amd64.c 2040 2008-02-21 08:57:23Z peter $");
+/*@unused@*/ RCSID("$Id: elf-x86-amd64.c 2148 2008-10-08 07:42:56Z peter $");
 
 #include <libyasm.h>
 #define YASM_OBJFMT_ELF_INTERNAL
@@ -156,6 +156,10 @@ elf_x86_amd64_map_reloc_info_to_type(elf_reloc_entry *reloc,
                     if (esym)
                         esym->type = STT_TLS;
                 }
+                /* Map PC-relative GOT to appropriate relocation */
+                if (reloc->rtype_rel &&
+                    elf_x86_amd64_ssyms[i].reloc == R_X86_64_GOT32)
+                    return (unsigned char) R_X86_64_GOTPCREL;
                 return (unsigned char) elf_x86_amd64_ssyms[i].reloc;
             }
         }
