@@ -88,25 +88,10 @@
        Will currently produce an error though the necessary means are provided
        by the Mach-O specification.
 
-  3) position independend coding (64 Bit)
-     Mach-O provides the relocation features X86_64_RELOC_GOT_LOAD and
-     X86_64_RELOC_GOT for C-compatible global offset tables. This IS NOT
-     implemented yet, sorry.
-
-  4) symbol naming for global and external symbols
-     BSD, Windows and MacOS-X use underscores for global symbols,
-     where ELF/Linux does not. This file contains simple means of adding
-     underscores to symbols by defining "AUTO_UNDERSCORE" but that
-     can be considered not more than a hack. For cross-platform coding,
-     use two symbols for your routines and referenced data.
-
 */
 
 #include <util.h>
-/*@unused@*/ RCSID("$Id: macho-objfmt.c 2035 2008-02-09 03:35:07Z peter $");
-
-/* optional: automatically prefix underscores to global exported symbols */
-/*#define AUTO_UNDERSCORE*/
+/*@unused@*/ RCSID("$Id: macho-objfmt.c 2143 2008-10-08 07:29:43Z peter $");
 
 #include <libyasm.h>
 
@@ -293,7 +278,6 @@ typedef struct macho_symrec_data {
     unsigned long index;        /* index in output order */
     yasm_intnum *value;         /* valid after writing symtable to file */
     unsigned long length;       /* length + 1 (plus auto underscore) */
-    int add_uscore;             /* add underscore (0/1) */
 } macho_symrec_data;
 
 
@@ -302,6 +286,8 @@ typedef struct yasm_objfmt_macho {
 
     long parse_scnum;           /* sect numbering in parser */
     int bits;                   /* 32 / 64 */
+
+    yasm_symrec *gotpcrel_sym;  /* ..gotpcrel */
 } yasm_objfmt_macho;
 
 
@@ -368,13 +354,17 @@ macho_objfmt_create_common(yasm_object *object, yasm_objfmt_module *module,
 
     /* Support x86 and amd64 machines of x86 arch */
     if (yasm__strcasecmp(yasm_arch_get_machine(object->arch), "x86") == 0 &&
-        (bits_pref == 0 || bits_pref == 32))
+        (bits_pref == 0 || bits_pref == 32)) {
         objfmt_macho->bits = 32;
-    else if (yasm__strcasecmp(yasm_arch_get_machine(object->arch),
+        objfmt_macho->gotpcrel_sym = NULL;
+    } else if (yasm__strcasecmp(yasm_arch_get_machine(object->arch),
                               "amd64") == 0 &&
-             (bits_pref == 0 || bits_pref == 64))
+             (bits_pref == 0 || bits_pref == 64)) {
         objfmt_macho->bits = 64;
-    else {
+        /* FIXME: misuse of NULL bytecode */
+        objfmt_macho->gotpcrel_sym =
+            yasm_symtab_define_label(object->symtab, "..gotpcrel", NULL, 0, 0);
+    } else {
         yasm_xfree(objfmt_macho);
         return NULL;
     }
@@ -496,9 +486,13 @@ macho_objfmt_output_value(yasm_value *value, unsigned char *buf,
             return 1;
         }
 
-        if (value->wrt) {
+        if (value->curpos_rel && objfmt_macho->gotpcrel_sym &&
+            value->wrt == objfmt_macho->gotpcrel_sym) {
+            reloc->type = X86_64_RELOC_GOT;
+            value->wrt = NULL;
+        } else if (value->wrt) {
             yasm_error_set(YASM_ERROR_TOO_COMPLEX,
-                           N_("macho: WRT not supported"));
+                           N_("macho: invalid WRT"));
             yasm_xfree(reloc);
             return 1;
         }
@@ -513,7 +507,11 @@ macho_objfmt_output_value(yasm_value *value, unsigned char *buf,
             } else {
                 /* Add in the offset plus value size to end up with 0. */
                 intn_plus = offset+destsize;
-                if (value->jump_target)
+                if (reloc->type == X86_64_RELOC_GOT) {
+                    /* XXX: This is a hack */
+                    if (offset >= 2 && buf[-2] == 0x8B)
+                        reloc->type = X86_64_RELOC_GOT_LOAD;
+                } else if (value->jump_target)
                     reloc->type = X86_64_RELOC_BRANCH;
                 else
                     reloc->type = X86_64_RELOC_SIGNED;
@@ -799,7 +797,7 @@ static int
 macho_objfmt_count_sym(yasm_symrec *sym, /*@null@*/ void *d)
 {
     /*@null@*/ macho_objfmt_output_info *info = (macho_objfmt_output_info *)d;
-    const char *name;
+    /*@only@*/ char *name;
     yasm_sym_vis vis = yasm_symrec_get_visibility(sym);
 
     assert(info != NULL);
@@ -816,19 +814,13 @@ macho_objfmt_count_sym(yasm_symrec *sym, /*@null@*/ void *d)
             sym_data->index = info->symindex;
             info->symindex++;
 
-            name = yasm_symrec_get_name(sym);   /*printf("%s\n",name); */
-            sym_data->add_uscore = 0;
-#ifdef AUTO_UNDERSCORE
-            if (vis & (YASM_SYM_EXTERN | YASM_SYM_COMMON | YASM_SYM_GLOBAL)) {
-                if (name[0] != '_')
-                    sym_data->add_uscore = 1;
-            }
-#endif
+            name = yasm_symrec_get_global_name(sym, info->object);
+            /*printf("%s\n",name); */
             /* name length + delimiter */
-            sym_data->length =
-                (unsigned long)strlen(name) + sym_data->add_uscore + 1;
+            sym_data->length = (unsigned long)strlen(name) + 1;
             info->strlength += sym_data->length;
             info->indx++;
+            yasm_xfree(name);
         }
     }
     return 0;
@@ -968,13 +960,13 @@ macho_objfmt_output_str(yasm_symrec *sym, /*@null@*/ void *d)
     if (info->all_syms ||
         vis & (YASM_SYM_GLOBAL | YASM_SYM_COMMON | YASM_SYM_EXTERN)) {
         if (0 == macho_objfmt_is_section_label(sym)) {
-            const char *name = yasm_symrec_get_name(sym);
+            /*@only@*/ char *name =
+                yasm_symrec_get_global_name(sym, info->object);
             size_t len = strlen(name);
 
             xsymd = yasm_symrec_get_data(sym, &macho_symrec_data_cb);
-            if (xsymd->add_uscore)
-                fputc('_', info->f);
             fwrite(name, len + 1, 1, info->f);
+            yasm_xfree(name);
         }
     }
     return 0;
@@ -1489,6 +1481,10 @@ static /*@observer@*/ /*@null@*/ yasm_symrec *
 macho_objfmt_get_special_sym(yasm_object *object, const char *name,
                              const char *parser)
 {
+    yasm_objfmt_macho *objfmt_macho = (yasm_objfmt_macho *)object->objfmt;
+    if (yasm__strcasecmp(name, "gotpcrel") == 0) {
+        return objfmt_macho->gotpcrel_sym;
+    }
     return NULL;
 }
 
