@@ -1,4 +1,4 @@
-/* $Id: gas-parser.h 1978 2007-09-21 16:59:14Z peter $
+/* $Id: gas-parser.h 2007 2007-11-03 04:27:35Z peter $
  * GAS-compatible parser header file
  *
  *  Copyright (C) 2005-2007  Peter Johnson
@@ -122,6 +122,10 @@ typedef struct yasm_parser_gas {
     YYCTYPE save_line[2][MAX_SAVED_LINE_LEN];
     int save_last;
 
+    /* Line data storage used in preproc_input(). */
+    char *line, *linepos;
+    size_t lineleft;
+
     yasm_scanner s;
     enum gas_parser_state state;
 
